@@ -57,37 +57,27 @@ inline void Array<N>::init(const std::array<size_t, N>& shape)
 }
 
 template <size_t N>
-template <class T>
-inline auto Array<N>::view_tensor2(T& var, size_t i) const
-{
-    return xt::adapt(
-        &var.flat(i * m_stride_tensor2),
-        m_stride_tensor2,
-        xt::no_ownership(),
-        std::array<size_t, 2>{m_ndim, m_ndim});
-}
-
-template <size_t N>
-template <class T>
-inline auto Array<N>::view_tensor4(T& var, size_t i) const
-{
-    return xt::adapt(
-        &var.flat(i * m_stride_tensor4),
-        m_stride_tensor4,
-        xt::no_ownership(),
-        std::array<size_t, 4>{m_ndim, m_ndim, m_ndim, m_ndim});
-}
-
-template <size_t N>
-inline std::array<size_t, N> Array<N>::shape() const
+inline const std::array<size_t, N>& Array<N>::shape() const
 {
     return m_shape;
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 2> Array<N>::O2() const
+inline const std::array<size_t, N + 2>& Array<N>::shape_tensor2() const
 {
-    xt::xtensor<double, N + 2> ret = xt::empty<double>(m_shape_tensor2);
+    return m_shape_tensor2;
+}
+
+template <size_t N>
+inline const std::array<size_t, N + 4>& Array<N>::shape_tensor4() const
+{
+    return m_shape_tensor4;
+}
+
+template <size_t N>
+inline array_type::tensor<double, N + 2> Array<N>::O2() const
+{
+    array_type::tensor<double, N + 2> ret = xt::empty<double>(m_shape_tensor2);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -98,9 +88,9 @@ inline xt::xtensor<double, N + 2> Array<N>::O2() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::O4() const
+inline array_type::tensor<double, N + 4> Array<N>::O4() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -111,9 +101,9 @@ inline xt::xtensor<double, N + 4> Array<N>::O4() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 2> Array<N>::I2() const
+inline array_type::tensor<double, N + 2> Array<N>::I2() const
 {
-    xt::xtensor<double, N + 2> ret = xt::empty<double>(m_shape_tensor2);
+    array_type::tensor<double, N + 2> ret = xt::empty<double>(m_shape_tensor2);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -124,9 +114,9 @@ inline xt::xtensor<double, N + 2> Array<N>::I2() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::II() const
+inline array_type::tensor<double, N + 4> Array<N>::II() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -137,9 +127,9 @@ inline xt::xtensor<double, N + 4> Array<N>::II() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::I4() const
+inline array_type::tensor<double, N + 4> Array<N>::I4() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -150,9 +140,9 @@ inline xt::xtensor<double, N + 4> Array<N>::I4() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::I4rt() const
+inline array_type::tensor<double, N + 4> Array<N>::I4rt() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -163,9 +153,9 @@ inline xt::xtensor<double, N + 4> Array<N>::I4rt() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::I4s() const
+inline array_type::tensor<double, N + 4> Array<N>::I4s() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
@@ -176,9 +166,9 @@ inline xt::xtensor<double, N + 4> Array<N>::I4s() const
 }
 
 template <size_t N>
-inline xt::xtensor<double, N + 4> Array<N>::I4d() const
+inline array_type::tensor<double, N + 4> Array<N>::I4d() const
 {
-    xt::xtensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
+    array_type::tensor<double, N + 4> ret = xt::empty<double>(m_shape_tensor4);
 
 #pragma omp parallel for
     for (size_t i = 0; i < m_size; ++i) {
