@@ -9,51 +9,17 @@ Implementation details (not part of public API).
 #ifndef GMATTENSOR_DETAIL_HPP
 #define GMATTENSOR_DETAIL_HPP
 
+#include "config.h"
+
 namespace GMatTensor {
 namespace detail {
 
-template <size_t RANK, class T>
-struct allocate
-{
-};
-
-template <size_t RANK, class EC, size_t N, xt::layout_type L, class Tag>
-struct allocate<RANK, xt::xtensor<EC, N, L, Tag>>
-{
-    using type = typename xt::xtensor<EC, RANK, L, Tag>;
-};
-
-#ifdef XTENSOR_FIXED_HPP
-template <size_t RANK, class EC, class S, xt::layout_type L>
-struct allocate<RANK, xt::xtensor_fixed<EC, S, L>>
-{
-    using type = typename xt::xtensor<EC, RANK, L>;
-};
-#endif
-
-#ifdef XTENSOR_FIXED_HPP
-template <size_t RANK, class EC, class S, xt::layout_type L, bool SH, class Tag>
-struct allocate<RANK, xt::xfixed_container<EC, S, L, SH, Tag>>
-{
-    using type = typename xt::xtensor<EC, RANK, L, Tag>;
-};
-#endif
-
-#ifdef PY_TENSOR_HPP
-template <size_t RANK, class EC, size_t N, xt::layout_type L>
-struct allocate<RANK, xt::pytensor<EC, N, L>>
-{
-    using type = typename xt::pytensor<EC, RANK, L>;
-};
-#endif
-
 // --------------------------
 // Array of 2nd-order tensors
 // --------------------------
 
 template <class T, size_t nd>
-struct impl_A2
-{
+struct impl_A2 {
     using value_type = typename T::value_type;
     using shape_type = typename T::shape_type;
     static_assert(xt::has_fixed_rank_t<T>::value, "Only fixed rank allowed.");
@@ -114,7 +80,7 @@ struct impl_A2
     {
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT0(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             ret.flat(i) = func(&A.flat(i * stride2));
         }
@@ -130,7 +96,7 @@ struct impl_A2
         using return_type = typename allocate<rank, T>::type;
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT0(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             ret.flat(i) = func(&A.flat(i * stride2));
         }
@@ -146,7 +112,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(B.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT0(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             ret.flat(i) = func(&A.flat(i * stride2), &B.flat(i * stride2));
         }
@@ -159,7 +125,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT0(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             ret.flat(i) = func(&A.flat(i * stride2), &B.flat(i * stride2));
         }
@@ -175,7 +141,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(B.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT2(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -188,7 +154,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT2(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -204,7 +170,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(B.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT4(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &B.flat(i * stride2), &ret.flat(i * stride4));
         }
@@ -217,7 +183,7 @@ struct impl_A2
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT4(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &B.flat(i * stride2), &ret.flat(i * stride4));
         }
@@ -232,7 +198,7 @@ struct impl_A2
     {
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT2(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -244,7 +210,7 @@ struct impl_A2
         using return_type = typename allocate<rank + 2, T>::type;
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT2(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -257,8 +223,7 @@ struct impl_A2
 // --------------------------
 
 template <class T, size_t nd>
-struct impl_A4
-{
+struct impl_A4 {
     using value_type = typename T::value_type;
     using shape_type = typename T::shape_type;
     static_assert(xt::has_fixed_rank_t<T>::value, "Only fixed rank allowed.");
@@ -319,7 +284,7 @@ struct impl_A4
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT4(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT2(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride4), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -332,7 +297,7 @@ struct impl_A4
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT4(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT2(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride4), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -348,7 +313,7 @@ struct impl_A4
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT4(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(ret, toShapeT4(A.shape())));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride4), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
@@ -361,7 +326,7 @@ struct impl_A4
         GMATTENSOR_ASSERT(xt::has_shape(A, toShapeT4(A.shape())));
         GMATTENSOR_ASSERT(xt::has_shape(B, toShapeT2(A.shape())));
         return_type ret = return_type::from_shape(toShapeT4(A.shape()));
-        #pragma omp parallel for
+#pragma omp parallel for
         for (size_t i = 0; i < toSizeT0(A.shape()); ++i) {
             func(&A.flat(i * stride4), &B.flat(i * stride2), &ret.flat(i * stride2));
         }
